@@ -4,6 +4,7 @@ import sys
 from cStringIO import StringIO
 
 import cola
+from cola import i18n
 from cola import core
 from cola import gitcfg
 from cola import gitcmds
@@ -13,6 +14,7 @@ from cola import cmdfactory
 from cola import difftool
 from cola import version
 from cola.diffparse import DiffParser
+from cola.models import selection
 
 _notifier = cola.notifier()
 _factory = cmdfactory.factory()
@@ -125,7 +127,7 @@ class AmendMode(Command):
             if os.path.exists(self.model.git.git_path('MERGE_HEAD')):
                 self.skip = True
                 _notifier.broadcast(signals.amend, False)
-                _notifier.broadcast(signals.information,
+                _factory.prompt_user(signals.information,
                                     'Oops! Unmerged',
                                     'You are in the middle of a merge.\n'
                                     'You cannot amend while merging.')
@@ -210,7 +212,7 @@ class ApplyPatches(Command):
         # Display a diffstat
         self.model.set_diff_text(diff_text)
 
-        _notifier.broadcast(signals.information,
+        _factory.prompt_user(signals.information,
                             'Patch(es) Applied',
                             '%d patch(es) applied:\n\n%s' %
                             (len(self.patches),
@@ -315,7 +317,7 @@ class Delete(Command):
                     os.remove(filename)
                     rescan=True
                 except:
-                    _notifier.broadcast(signals.information,
+                    _factory.prompt_user(signals.information,
                                         'Error'
                                         'Deleting "%s" failed.' % filename)
         if rescan:
@@ -419,7 +421,9 @@ class Difftool(Command):
         args = []
         if self.staged and not self.model.read_only():
             args.append('--cached')
-        args.extend([self.model.head, '--'])
+        if self.model.head != 'HEAD':
+            args.append(self.model.head)
+        args.append('--')
         args.extend(self.filenames)
         difftool.launch(args)
 
@@ -540,14 +544,91 @@ class ReviewBranchMode(Command):
     def __init__(self, branch):
         Command.__init__(self, update=True)
         self.new_mode = self.model.mode_review
-        self.new_head = gitcmds.merge_base_to(branch)
+        self.new_head = gitcmds.merge_base_parent(branch)
         self.new_diff_text = ''
 
 
+class RunConfigAction(Command):
+    """Run a user-configured action, typically from the "Tools" menu"""
+    def __init__(self, name):
+        Command.__init__(self)
+        self.name = name
+        self.model = cola.model()
+
+    def do(self):
+        for env in ('FILENAME', 'REVISION', 'ARGS'):
+            try:
+                del os.environ[env]
+            except KeyError:
+                pass
+        rev = None
+        args = None
+        opts = _config.get_guitool_opts(self.name)
+        cmd = opts.get('cmd')
+        if 'title' not in opts:
+            opts['title'] = cmd
+
+        if 'prompt' not in opts or opts.get('prompt') is True:
+            prompt = i18n.gettext('Are you sure you want to run %s?') % cmd
+            opts['prompt'] = prompt
+
+        if opts.get('needsfile'):
+            filename = selection.filename()
+            if not filename:
+                _factory.prompt_user(signals.information,
+                                     'Please select a file',
+                                     '"%s" requires a selected file' % cmd)
+                return
+            os.environ['FILENAME'] = utils.shell_quote(filename)
+
+
+        if opts.get('revprompt') or opts.get('argprompt'):
+            while True:
+                ok = _factory.prompt_user(signals.run_config_action, cmd, opts)
+                if not ok:
+                    return
+                rev = opts.get('revision')
+                args = opts.get('args')
+                if opts.get('revprompt') and not rev:
+                    msg = ('Invalid revision:\n\n'
+                           'Revision expression is empty')
+                    title = 'Oops!'
+                    _factory.prompt_user(signals.information, title, msg)
+                    continue
+                break
+
+        elif opts.get('confirm'):
+            title = os.path.expandvars(opts.get('title'))
+            prompt = os.path.expandvars(opts.get('prompt'))
+            if not _factory.prompt_user(signals.question, title, prompt):
+                return
+        if rev:
+            os.environ['REVISION'] = rev
+        if args:
+            os.environ['ARGS'] = args
+        title = os.path.expandvars(cmd)
+        _notifier.broadcast(signals.log_cmd, 0, 'running: ' + title)
+        cmd = ['sh', '-c', cmd]
+
+        if opts.get('noconsole'):
+            status, out, err = utils.run_command(cmd, flag_error=False)
+        else:
+            status, out, err = _factory.prompt_user(signals.run_command,
+                                                    title, cmd)
+
+        _notifier.broadcast(signals.log_cmd, status,
+                            'stdout: %s\nstatus: %s\nstderr: %s' %
+                                (out.rstrip(), status, err.rstrip()))
+
+        if not opts.get('norescan'):
+            self.model.update_status()
+        return status
+
+
 class ShowUntracked(Command):
     """Show an untracked file."""
     # We don't actually do anything other than set the mode right now.
-    # We could probably check the mimetype for the file and handle things
+    # TODO check the mimetype for the file and handle things
     # generically.
     def __init__(self, filenames):
         Command.__init__(self)
@@ -624,8 +705,7 @@ class Unstage(Command):
     def do(self):
         msg = 'Unstaging: %s' % (', '.join(self.paths))
         _notifier.broadcast(signals.log_cmd, 0, msg)
-        gitcmds.unstage_paths(self.paths)
-        self.model.update_status()
+        self.model.unstage_paths(self.paths)
 
 
 class UnstageAll(Command):
@@ -724,6 +804,7 @@ def register():
         signals.rescan: Rescan,
         signals.reset_mode: ResetMode,
         signals.review_branch_mode: ReviewBranchMode,
+        signals.run_config_action: RunConfigAction,
         signals.show_untracked: ShowUntracked,
         signals.stage: Stage,
         signals.stage_modified: StageModified,
