@@ -13,7 +13,9 @@ from cola import utils
 from cola import gitcmd
 from cola import gitcfg
 from cola import gitcmds
-from cola.models.observable import ObservableModel
+from cola.compat import set
+from cola import serializer
+from cola.models.observable import ObservableModel, OMSerializer
 
 # Static GitConfig instance
 _config = gitcfg.instance()
@@ -29,6 +31,10 @@ def model():
     return _instance
 
 
+class MainSerializer(OMSerializer):
+    def post_decode_hook(self):
+        OMSerializer.post_decode_hook(self)
+        self.obj.generate_remote_helpers()
 
 class MainModel(ObservableModel):
     """Provides a friendly wrapper for doing common git operations."""
@@ -54,6 +60,9 @@ class MainModel(ObservableModel):
     # Modes where we can checkout files from the $head
     modes_undoable = (mode_none, mode_index, mode_worktree)
 
+    unstaged = property(lambda self: self.modified + self.unmerged + self.untracked)
+    """An aggregate of the modified, unmerged, and untracked file lists."""
+
     def __init__(self, cwd=None):
         """Reads git repository settings and sets several methods
         so that they refer to the git module.  This object
@@ -82,10 +91,10 @@ class MainModel(ObservableModel):
         self.commitmsg = ''
         self.modified = []
         self.staged = []
-        self.unstaged = []
         self.untracked = []
         self.unmerged = []
         self.upstream_changed = []
+        self.submodules = set()
 
         #####################################################
         # Refs
@@ -103,6 +112,10 @@ class MainModel(ObservableModel):
         if cwd:
             self.use_worktree(cwd)
 
+        #####################################################
+        # Dag
+        self._commits = []
+
     def read_only(self):
         return self.mode in self.modes_read_only
 
@@ -156,7 +169,7 @@ class MainModel(ObservableModel):
             'merge_keepbackup': True,
             'diff_tool': os.getenv('GIT_DIFF_TOOL', 'xxdiff'),
             'merge_tool': os.getenv('GIT_MERGE_TOOL', 'xxdiff'),
-            'gui_editor': os.getenv('EDITOR', 'gvim'),
+            'gui_editor': os.getenv('VISUAL', os.getenv('EDITOR', 'gvim')),
             'gui_historybrowser': 'gitk',
         }
 
@@ -250,7 +263,8 @@ class MainModel(ObservableModel):
         return self.config_set(param, value, local=is_local)
 
     def editor(self):
-        return self.gui_config('editor')
+        app = self.gui_config('editor')
+        return {'vim': 'gvim'}.get(app, app)
 
     def history_browser(self):
         return self.gui_config('historybrowser')
@@ -278,48 +292,56 @@ class MainModel(ObservableModel):
 
     def prev_commitmsg(self):
         """Queries git for the latest commit message."""
-        return core.decode(self.git.log('-1', pretty='format:%s%n%n%b'))
+        return core.decode(self.git.log('-1', no_color=True, pretty='format:%s%n%n%b'))
 
     def update_status(self):
         # Give observers a chance to respond
         self.notify_message_observers(self.message_about_to_update)
         # This allows us to defer notification until the
         # we finish processing data
-        staged_only = self.read_only()
-        head = self.head
-        notify_enabled = self.notification_enabled
         self.notification_enabled = False
 
+        self._update_files()
+        self._update_refs()
+        self._update_branches_and_tags()
+        self._update_branch_heads()
+
+        # Re-enable notifications and emit changes
+        self.notification_enabled = True
+        self.notify_observers('staged', 'unstaged')
+        self.notify_message_observers(self.message_updated)
+
+        self.read_font_sizes()
+
+    def _update_files(self, worktree_only=False):
+        staged_only = self.read_only()
+        state = gitcmds.worktree_state_dict(head=self.head,
+                                            staged_only=staged_only)
+        self.staged = state.get('staged', [])
+        self.modified = state.get('modified', [])
+        self.unmerged = state.get('unmerged', [])
+        self.untracked = state.get('untracked', [])
+        self.submodules = state.get('submodules', set())
+        self.upstream_changed = state.get('upstream_changed', [])
+
+    def _update_refs(self):
+        self.set_remotes(self.git.remote().splitlines())
+        self.set_revision('')
+        self.set_local_branch('')
+        self.set_remote_branch('')
+
+
+    def _update_branch_heads(self):
         # Set these early since they are used to calculate 'upstream_changed'.
         self.set_trackedbranch(gitcmds.tracked_branch())
         self.set_currentbranch(gitcmds.current_branch())
 
-        (self.staged,
-         self.modified,
-         self.unmerged,
-         self.untracked,
-         self.upstream_changed) = gitcmds.worktree_state(head=head,
-                                                staged_only=staged_only)
-        # NOTE: the model's unstaged list holds an aggregate of the
-        # the modified, unmerged, and untracked file lists.
-        self.set_unstaged(self.modified + self.unmerged + self.untracked)
-        self.set_remotes(self.git.remote().splitlines())
-
+    def _update_branches_and_tags(self):
         local_branches, remote_branches, tags = gitcmds.all_refs(split=True)
         self.set_local_branches(local_branches)
         self.set_remote_branches(remote_branches)
         self.set_tags(tags)
 
-        self.set_revision('')
-        self.set_local_branch('')
-        self.set_remote_branch('')
-        # Re-enable notifications and emit changes
-        self.notification_enabled = notify_enabled
-
-        self.read_font_sizes()
-        self.notify_observers('staged', 'unstaged')
-        self.notify_message_observers(self.message_updated)
-
     def read_font_sizes(self):
         """Read font sizes from the configuration."""
         value = self.cola_config('fontdiff')
@@ -422,7 +444,7 @@ class MainModel(ObservableModel):
             try:
                 k, v = line.split('=', 1)
             except:
-                # the user has an invalid entry in their git config
+                # value-less entry in .gitconfig
                 continue
             v = core.decode(v)
             k = k.replace('.','_') # git -> model
@@ -555,11 +577,6 @@ class MainModel(ObservableModel):
         else:
             return pstr
 
-    def describe(self, revid, descr):
-        version = self.git.describe(revid, tags=True, always=True,
-                                    abbrev=4)
-        return version + ' - ' + descr
-
     def is_commit_published(self):
         head = self.git.rev_parse('HEAD')
         return bool(self.git.branch(r=True, contains=head))
@@ -576,11 +593,35 @@ class MainModel(ObservableModel):
         """Stages add/removals to git."""
         add = []
         remove = []
+        sset = set(self.staged)
+        mset = set(self.modified)
+        umset = set(self.unmerged)
+        utset = set(self.untracked)
+        dirs = bool([p for p in paths if os.path.isdir(core.encode(p))])
+
+        if not dirs:
+            self.notify_message_observers(self.message_about_to_update)
+
         for path in set(paths):
+            if not os.path.isdir(core.encode(path)) and path not in sset:
+                self.staged.append(path)
+            if path in umset:
+                self.unmerged.remove(path)
+            if path in mset:
+                self.modified.remove(path)
+            if path in utset:
+                self.untracked.remove(path)
             if os.path.exists(core.encode(path)):
                 add.append(path)
             else:
                 remove.append(path)
+
+        if dirs:
+            self.notify_message_observers(self.message_about_to_update)
+
+        elif add or remove:
+            self.staged.sort()
+
         # `git add -u` doesn't work on untracked files
         if add:
             self.git.add('--', *add)
@@ -588,10 +629,48 @@ class MainModel(ObservableModel):
         # from the index.   We use `git add -u` for that.
         if remove:
             self.git.add('--', u=True, *remove)
-        self.update_status()
+
+        if dirs:
+            self._update_files()
+
+        self.notify_message_observers(self.message_updated)
+
+    def unstage_paths(self, paths):
+        self.notify_message_observers(self.message_about_to_update)
+
+        staged_set = set(self.staged)
+        gitcmds.unstage_paths(paths)
+        all_paths_set = set(gitcmds.all_files())
+        modified = []
+        untracked = []
+
+        cur_modified_set = set(self.modified)
+        cur_untracked_set = set(self.untracked)
+
+        for path in paths:
+            if path in staged_set:
+                self.staged.remove(path)
+                if path in all_paths_set:
+                    if path not in cur_modified_set:
+                        modified.append(path)
+                else:
+                    if path not in cur_untracked_set:
+                        untracked.append(path)
+
+        if modified:
+            self.modified.extend(modified)
+            self.modified.sort()
+
+        if untracked:
+            self.untracked.extend(untracked)
+            self.untracked.sort()
+
+        self.notify_message_observers(self.message_updated)
 
     def getcwd(self):
         """If we've chosen a directory then use it, otherwise os.getcwd()."""
         if self.directory:
             return self.directory
         return os.getcwd()
+
+serializer.handlers[MainModel] = MainSerializer
