@@ -1,6 +1,8 @@
 import os
 from PyQt4 import QtCore
 
+from cola.compat import set
+
 debug = os.environ.get('COLA_NOTIFIER_DEBUG', False)
 _instance = None
 def notifier():
@@ -29,9 +31,6 @@ class Notifier(object):
             for fxn in subscribers:
                 fxn(*args, **opts)
 
-    def listen(self, signal, callback):
-        self.connect(signal, callback)
-
     def connect(self, signal, callback):
         subscribers = self.channels.setdefault(signal, set())
         subscribers.add(callback)
