@@ -19,7 +19,7 @@ def logger():
     global _logger
     if not _logger:
         _logger = cola.views.log.LogView()
-        cola.notifier().listen(signals.log_cmd, _logger.log)
+        cola.notifier().connect(signals.log_cmd, _logger.log)
     return _logger
 
 
@@ -84,7 +84,7 @@ def information(title, message=None):
     parent = QtGui.QApplication.instance().activeWindow()
     QtGui.QMessageBox.information(parent, title, message)
 # Register globally with the notifier
-cola.notifier().listen(signals.information, information)
+cola.notifier().connect(signals.information, information)
 
 def selected_treeitem(tree_widget):
     """Returns a(id_number, is_selected) for a QTreeWidget."""
@@ -295,9 +295,18 @@ def set_diff_font(widget):
     widget.blockSignals(block)
 
 
-def add_close_acction(widget):
+def add_close_action(widget):
     """Adds a Ctrl+w close action to a widget."""
     action = QtGui.QAction(widget.tr('Close...'), widget)
     action.setShortcut('Ctrl+w')
     widget.addAction(action)
     widget.connect(action, SIGNAL('triggered()'), widget.close)
+
+
+def center_on_screen(widget):
+    """Move widget to the center of the default screen"""
+    desktop = QtGui.QApplication.instance().desktop()
+    rect = desktop.screenGeometry(QtGui.QCursor().pos())
+    cy = rect.height()/2
+    cx = rect.width()/2
+    widget.move(cx - widget.width()/2, cy - widget.height()/2)
