@@ -31,10 +31,13 @@ mergetool = 'mergetool'
 mode = 'mode'
 modified_summary = 'modified_summary'
 open_repo = 'open_repo'
+question = 'question'
 redo = 'redo'
 rescan = 'rescan'
 reset_mode = 'reset_mode'
 review_branch_mode = 'review_branch_mode'
+run_config_action = 'run_config_action'
+run_command = 'run_command'
 show_untracked = 'show_untracked'
 stage = 'stage'
 stage_diffs = 'stage_diffs'
