@@ -16,8 +16,11 @@ from cola import qtutils
 from cola import settings
 from cola import signals
 from cola import resources
+from cola.compat import set
 from cola.qtutils import SLOT
+from cola.views import dag
 from cola.views import about
+from cola.views import actions as actionsmod
 from cola.views.syntax import DiffSyntaxHighlighter
 from cola.views.mainwindow import MainWindow
 from cola.controllers import classic
@@ -30,12 +33,7 @@ from cola.controllers.createbranch import create_new_branch
 from cola.controllers.merge import local_merge
 from cola.controllers.merge import abort_merge
 from cola.controllers.options import update_options
-from cola.controllers.util import choose_from_combo
-from cola.controllers.util import choose_from_list
-from cola.controllers.remote import remote_action
-from cola.controllers.repobrowser import browse_git_branch
 from cola.controllers.stash import stash
-from cola.controllers.selectcommits import select_commits
 
 class MainView(MainWindow):
     """The main cola interface."""
@@ -77,10 +75,10 @@ class MainView(MainWindow):
                                         self._update_view)
 
         # Listen for text and amend messages
-        cola.notifier().listen(signals.diff_text, self.set_display)
-        cola.notifier().listen(signals.mode, self._mode_changed)
-        cola.notifier().listen(signals.inotify, self._inotify_enabled)
-        cola.notifier().listen(signals.amend, self.amend_checkbox.setChecked)
+        cola.notifier().connect(signals.diff_text, self.set_display)
+        cola.notifier().connect(signals.mode, self._mode_changed)
+        cola.notifier().connect(signals.inotify, self._inotify_enabled)
+        cola.notifier().connect(signals.amend, self.amend_checkbox.setChecked)
 
         # Broadcast the amend mode
         self.connect(self.amend_checkbox, SIGNAL('toggled(bool)'),
@@ -94,46 +92,46 @@ class MainView(MainWindow):
         self._connect_button(self.stage_button, self.stage)
         self._connect_button(self.unstage_button, self.unstage)
         self._connect_button(self.commit_button, self.commit)
-        self._connect_button(self.fetch_button, self.fetch)
-        self._connect_button(self.push_button, self.push)
-        self._connect_button(self.pull_button, self.pull)
+        self._connect_button(self.fetch_button, guicmds.fetch_slot(self))
+        self._connect_button(self.push_button, guicmds.push_slot(self))
+        self._connect_button(self.pull_button, guicmds.pull_slot(self))
         self._connect_button(self.stash_button, stash)
 
         # Menu actions
         actions = (
             (self.menu_quit, self.close),
             (self.menu_branch_compare, compare.branch_compare),
-            (self.menu_branch_diff, self.branch_diff),
-            (self.menu_branch_review, self.review_branch),
-            (self.menu_browse_branch, self.browse_current),
-            (self.menu_browse_other_branch, self.browse_other),
-            (self.menu_browse_commits, self.browse_commits),
+            (self.menu_branch_diff, guicmds.branch_diff),
+            (self.menu_branch_review, guicmds.review_branch),
+            (self.menu_browse_branch, guicmds.browse_current),
+            (self.menu_browse_other_branch, guicmds.browse_other),
+            (self.menu_browse_commits, guicmds.browse_commits),
             (self.menu_create_tag, createtag.create_tag),
             (self.menu_create_branch, create_new_branch),
-            (self.menu_checkout_branch, self.checkout_branch),
-            (self.menu_delete_branch, self.branch_delete),
-            (self.menu_rebase_branch, self.rebase),
+            (self.menu_checkout_branch, guicmds.checkout_branch),
+            (self.menu_delete_branch, guicmds.branch_delete),
+            (self.menu_rebase_branch, guicmds.rebase),
             (self.menu_clone_repo, guicmds.clone_repo),
             (self.menu_commit_compare, compare.compare),
             (self.menu_commit_compare_file, compare.compare_file),
-            (self.menu_cherry_pick, self.cherry_pick),
-            (self.menu_diff_expression, self.diff_expression),
-            (self.menu_diff_branch, self.diff_branch),
-            (self.menu_export_patches, self.export_patches),
+            (self.menu_cherry_pick, guicmds.cherry_pick),
+            (self.menu_diff_expression, guicmds.diff_expression),
+            (self.menu_diff_branch, guicmds.diff_branch),
+            (self.menu_export_patches, guicmds.export_patches),
             (self.menu_help_about, about.launch_about_dialog),
             (self.menu_help_docs,
                 lambda: self.model.git.web__browse(resources.html_docs())),
-            (self.menu_load_commitmsg, self.load_commitmsg),
+            (self.menu_load_commitmsg, guicmds.load_commitmsg_slot(self)),
             (self.menu_load_commitmsg_template,
                 SLOT(signals.load_commit_template)),
             (self.menu_manage_bookmarks, manage_bookmarks),
             (self.menu_save_bookmark, save_bookmark),
             (self.menu_merge_local, local_merge),
             (self.menu_merge_abort, abort_merge),
-            (self.menu_open_repo, self.open_repo),
+            (self.menu_open_repo, guicmds.open_repo_slot(self)),
             (self.menu_options, update_options),
             (self.menu_rescan, SLOT(signals.rescan)),
-            (self.menu_search_grep, self.grep),
+            (self.menu_search_grep, guicmds.grep),
             (self.menu_search_revision, smod.search(smod.REVISION_ID)),
             (self.menu_search_revision_range, smod.search(smod.REVISION_RANGE)),
             (self.menu_search_message, smod.search(smod.MESSAGE)),
@@ -159,10 +157,18 @@ class MainView(MainWindow):
             (self.menu_undo, self.commitmsg.undo),
             (self.menu_redo, self.commitmsg.redo),
             (self.menu_classic, classic.cola_classic),
+            (self.menu_dag, dag.git_dag),
         )
         for menu, callback in actions:
             self.connect(menu, SIGNAL('triggered()'), callback)
 
+        # Install UI wrappers for command objects
+        actionsmod.install_command_wrapper(self)
+        guicmds.install_command_wrapper(self)
+
+        # Install .gitconfig-defined actions
+        actionsmod.install_config_actions(self.actions_menu)
+
         # Install diff shortcut keys for stage/unstage
         self.display_text.keyPressEvent = self.diff_key_press_event
         self.display_text.contextMenuEvent = self.diff_context_menu_event
@@ -186,8 +192,12 @@ class MainView(MainWindow):
 
     def _update_view(self):
         """Update the title with the current branch and directory name."""
-        title = '%s [%s]' % (self.model.project,
-                             self.model.currentbranch)
+        branch = self.model.currentbranch
+        curdir = os.getcwd()
+        msg = 'Repository: %s\nBranch: %s' % (curdir, branch)
+        self.commitdockwidget.setToolTip(msg)
+
+        title = '%s [%s]' % (self.model.project, branch)
         if self.mode in (self.model.mode_diff, self.model.mode_diff_expr):
             title += ' *** diff mode***'
         elif self.mode == self.model.mode_review:
@@ -229,7 +239,7 @@ class MainView(MainWindow):
         scrollbar = self.display_text.verticalScrollBar()
         scrollvalue = scrollbar.value()
         if text is not None:
-            self.display_text.setText(text)
+            self.display_text.setPlainText(text)
             scrollbar.setValue(scrollvalue)
 
     def action_cut(self):
@@ -309,66 +319,11 @@ class MainView(MainWindow):
         state['windowstate'] = unicode(windowstate.toBase64().data())
         return state
 
-    def review_branch(self):
-        """Diff against an arbitrary revision, branch, tag, etc."""
-        branch = choose_from_combo('Select Branch, Tag, or Commit-ish',
-                                   self.model.all_branches() +
-                                   self.model.tags)
-        if not branch:
-            return
-        cola.notifier().broadcast(signals.review_branch_mode, branch)
-
-    def branch_diff(self):
-        """Diff against an arbitrary revision, branch, tag, etc."""
-        branch = choose_from_combo('Select Branch, Tag, or Commit-ish',
-                                   ['HEAD^'] +
-                                   self.model.all_branches() +
-                                   self.model.tags)
-        if not branch:
-            return
-        cola.notifier().broadcast(signals.diff_mode, branch)
-
-    def diff_expression(self):
-        """Diff using an arbitrary expression."""
-        expr = choose_from_combo('Enter Diff Expression',
-                                 self.model.all_branches() +
-                                 self.model.tags)
-        if not expr:
-            return
-        cola.notifier().broadcast(signals.diff_expr_mode, expr)
-
-
-    def diff_branch(self):
-        """Launches a diff against a branch."""
-        branch = choose_from_combo('Select Branch, Tag, or Commit-ish',
-                                   ['HEAD^'] +
-                                   self.model.all_branches() +
-                                   self.model.tags)
-        if not branch:
-            return
-        zfiles_str = self.model.git.diff(branch, name_only=True,
-                                         no_color=True,
-                                         z=True).rstrip('\0')
-        files = zfiles_str.split('\0')
-        filename = choose_from_list('Select File', files)
-        if not filename:
-            return
-        cola.notifier().broadcast(signals.branch_mode, branch, filename)
-
     def _load_gui_state(self):
         """Restores the gui from the preferences file."""
         state = settings.SettingsManager.gui_state(self)
         self.import_state(state)
 
-    def load_commitmsg(self):
-        """Load a commit message from a file."""
-        filename = qtutils.open_dialog(self,
-                                       'Load Commit Message...',
-                                       self.model.getcwd())
-        if filename:
-            cola.notifier().broadcast(signals.load_commit_message, filename)
-
-
     def diff_key_press_event(self, event):
         """Handle shortcut keys in the diff view."""
         if event.key() != QtCore.Qt.Key_H and event.key() != QtCore.Qt.Key_S:
@@ -466,7 +421,13 @@ class MainView(MainWindow):
         staged, modified, unmerged, untracked = cola.selection()
 
         if self.mode == self.model.mode_worktree:
-            if modified:
+            if modified and modified[0] in cola.model().submodules:
+                menu.addAction(self.tr('Stage'),
+                               SLOT(signals.stage, modified))
+                menu.addAction(self.tr('Launch git-cola'),
+                               SLOT(signals.open_repo,
+                                    os.path.abspath(modified[0])))
+            elif modified:
                 menu.addAction(self.tr('Stage &Hunk For Commit'),
                                self.stage_hunk)
                 menu.addAction(self.tr('Stage &Selected Lines'),
@@ -478,32 +439,32 @@ class MainView(MainWindow):
                                self.undo_selection)
 
         elif self.mode == self.model.mode_index:
-            menu.addAction(self.tr('Unstage &Hunk From Commit'), self.unstage_hunk)
-            menu.addAction(self.tr('Unstage &Selected Lines'), self.unstage_hunk_selection)
+            if staged and staged[0] in cola.model().submodules:
+                menu.addAction(self.tr('Unstage'),
+                               SLOT(signals.unstage, staged))
+                menu.addAction(self.tr('Launch git-cola'),
+                               SLOT(signals.open_repo,
+                                    os.path.abspath(staged[0])))
+            else:
+                menu.addAction(self.tr('Unstage &Hunk From Commit'),
+                               self.unstage_hunk)
+                menu.addAction(self.tr('Unstage &Selected Lines'),
+                               self.unstage_hunk_selection)
 
         elif self.mode == self.model.mode_branch:
-            menu.addAction(self.tr('Apply Diff to Work Tree'), self.stage_hunk)
-            menu.addAction(self.tr('Apply Diff Selection to Work Tree'), self.stage_hunk_selection)
+            menu.addAction(self.tr('Apply Diff to Work Tree'),
+                           self.stage_hunk)
+            menu.addAction(self.tr('Apply Diff Selection to Work Tree'),
+                           self.stage_hunk_selection)
 
         elif self.mode == self.model.mode_grep:
-            menu.addAction(self.tr('Go Here'), self.goto_grep)
+            menu.addAction(self.tr('Go Here'),
+                           lambda: guicmds.goto_grep(self.selected_line()))
 
         menu.addSeparator()
         menu.addAction(self.tr('Copy'), self.copy_display)
         return menu
 
-    def fetch(self):
-        """Launch the 'fetch' remote dialog."""
-        remote_action(self, 'fetch')
-
-    def push(self):
-        """Launch the 'push' remote dialog."""
-        remote_action(self, 'push')
-
-    def pull(self):
-        """Launch the 'pull' remote dialog."""
-        remote_action(self, 'pull')
-
     def commit(self):
         """Attempt to create a commit from the index and commit message."""
         #self.reset_mode()
@@ -544,99 +505,6 @@ class MainView(MainWindow):
         # Perform the commit
         cola.notifier().broadcast(signals.commit, amend, msg)
 
-    def grep(self):
-        """Prompt and use 'git grep' to find the content."""
-        # This should be a command in cola.commands.
-        txt, ok = qtutils.prompt('grep')
-        if not ok:
-            return
-        cola.notifier().broadcast(signals.grep, txt)
-
-    def goto_grep(self):
-        """Called when Search -> Grep's right-click 'goto' action."""
-        line = self.selected_line()
-        filename, line_number, contents = line.split(':', 2)
-        filename = core.encode(filename)
-        cola.notifier().broadcast(signals.edit, [filename], line_number=line_number)
-
-    def open_repo(self):
-        """Spawn a new cola session."""
-        dirname = qtutils.opendir_dialog(self,
-                                         'Open Git Repository...',
-                                         self.model.getcwd())
-        if not dirname:
-            return
-        cola.notifier().broadcast(signals.open_repo, dirname)
-
-    def cherry_pick(self):
-        """Launch the 'Cherry-Pick' dialog."""
-        revs, summaries = gitcmds.log_helper(all=True)
-        commits = select_commits('Cherry-Pick Commit',
-                                 revs, summaries, multiselect=False)
-        if not commits:
-            return
-        cola.notifier().broadcast(signals.cherry_pick, commits)
-
-    def browse_commits(self):
-        """Launch the 'Browse Commits' dialog."""
-        revs, summaries = gitcmds.log_helper(all=True)
-        select_commits('Browse Commits', revs, summaries)
-
-    def export_patches(self):
-        """Run 'git format-patch' on a list of commits."""
-        revs, summaries = gitcmds.log_helper()
-        to_export = select_commits('Export Patches', revs, summaries)
-        if not to_export:
-            return
-        to_export.reverse()
-        revs.reverse()
-        cola.notifier().broadcast(signals.format_patch, to_export, revs)
-
-    def browse_current(self):
-        """Launch the 'Browse Current Branch' dialog."""
-        browse_git_branch(gitcmds.current_branch())
-
-    def browse_other(self):
-        """Prompt for a branch and inspect content at that point in time."""
-        # Prompt for a branch to browse
-        branch = choose_from_combo('Browse Revision...', gitcmds.all_refs())
-        if not branch:
-            return
-        # Launch the repobrowser
-        browse_git_branch(branch)
-
-    def branch_create(self):
-        """Launch the 'Create Branch' dialog."""
-        create_new_branch()
-
-    def branch_delete(self):
-        """Launch the 'Delete Branch' dialog."""
-        branch = choose_from_combo('Delete Branch',
-                                   self.model.local_branches)
-        if not branch:
-            return
-        cola.notifier().broadcast(signals.delete_branch, branch)
-
-    def checkout_branch(self):
-        """Launch the 'Checkout Branch' dialog."""
-        branch = choose_from_combo('Checkout Branch',
-                                   self.model.local_branches)
-        if not branch:
-            return
-        cola.notifier().broadcast(signals.checkout_branch, branch)
-
-    def rebase(self):
-        """Rebase onto a branch."""
-        branch = choose_from_combo('Rebase Branch',
-                                   self.model.all_branches())
-        if not branch:
-            return
-        #TODO cmd
-        status, output = self.model.git.rebase(branch,
-                                               with_stderr=True,
-                                               with_status=True)
-        qtutils.log(status, output)
-
     def dragEnterEvent(self, event):
         """Accepts drops"""
         MainWindow.dragEnterEvent(self, event)
